//---------------------------------------------------------------------------
#include "SelfThinning.h"
//---------------------------------------------------------------------------
#include "MortalityOrg.h"
#include "ParsingFunctions.h"
#include "TreePopulation.h"
#include "SimManager.h"
#include "ModelMath.h"
#include <math.h>

////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
clSelfThinMort::clSelfThinMort(clSimManager *p_oSimManager) :
clWorkerBase( p_oSimManager ), clBehaviorBase( p_oSimManager ),
clMortalityBase( p_oSimManager ) { 
 try {

   strcpy(m_cNameString, "selfthinmortshell");

   mp_fSelfThinSlope = NULL;
   mp_fSelfThinIntercept = NULL;
   mp_fSelfThinMaxDbh = NULL;

 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clSelfThinMort::clSelfThinMort");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/*///////////////////////////////////////////////////////////////////////////
clSelfThinMort::~clSelfThinMort() {
 delete[] mp_fSelfThinSlope;
 delete[] mp_fSelfThinIntercept;
 delete[] mp_fSelfThinMaxDbh;
}
//---------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
SetNameData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/*///////////////////////////////////////////////////////////////////////////
void clSelfThinMort::SetNameData(char *cNameString) {
 if (strcmp(cNameString, "adultselfthin") == 0)
   m_bIsAdult = true;
 else
   m_bIsAdult = false;
}
//---------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
DoShellSetup()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/*///////////////////////////////////////////////////////////////////////////
void clSelfThinMort::DoShellSetup(xercesc::DOMDocument *p_oDoc) {
 try {
   clPopulationBase *p_oTemp =
           mp_oSimManager->GetPopulationObject("treepopulation");
   clTreePopulation *p_oPop = (clTreePopulation*) p_oTemp;
   DOMElement *p_oElement = p_oDoc->getDocumentElement();
   floatVal *p_fTempValues;  //for getting species-specific values
   short int iNumSpecies = mp_oMortalityOrg->GetNumberOfSpecies(),
             i; //loop counter

   m_fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();

   //Declare the temp array and populate it with the species to which this
   //behavior applies
   p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
   for (i = 0; i < m_iNumBehaviorSpecies; i++)
     p_fTempValues[i].code = mp_iWhatSpecies[i];

   //Declare the arrays for holding the variables
   mp_fSelfThinSlope = new float[iNumSpecies];
   mp_fSelfThinIntercept = new float[iNumSpecies];
   if (m_bIsAdult)
     mp_fSelfThinMaxDbh = new float[iNumSpecies];

   //Capture the values from the parameter file

   //Self-thinning slope
   if (m_bIsAdult)
     FillSpeciesSpecificValue(p_oElement, "mo_adultSelfThinSlope", "mo_astsVal",
                    p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   else
     FillSpeciesSpecificValue(p_oElement, "mo_juvSelfThinSlope", "mo_jstsVal",
                    p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   //Transfer to the appropriate array buckets
   for (i = 0; i < m_iNumBehaviorSpecies; i++)
     mp_fSelfThinSlope[p_fTempValues[i].code] = p_fTempValues[i].val;


   //Self-thinning intercept
   if (m_bIsAdult)
     FillSpeciesSpecificValue(p_oElement, "mo_adultSelfThinIntercept",
             "mo_astiVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   else
     FillSpeciesSpecificValue(p_oElement, "mo_juvSelfThinIntercept",
             "mo_jstiVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
   //Transfer to the appropriate array buckets
   for (i = 0; i < m_iNumBehaviorSpecies; i++)
     mp_fSelfThinIntercept[p_fTempValues[i].code] = p_fTempValues[i].val;


   //Max self-thinning dbh
   if (m_bIsAdult) {
     FillSpeciesSpecificValue(p_oElement, "mo_adultSelfThinMaxDbh",
             "mo_astmdVal", p_fTempValues, m_iNumBehaviorSpecies, p_oPop, true);
     //Transfer to the appropriate array buckets
     for (i = 0; i < m_iNumBehaviorSpecies; i++)
       mp_fSelfThinMaxDbh[p_fTempValues[i].code] = p_fTempValues[i].val;
   }
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clSelfThinMort::DoShellSetup");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// DoMort()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
bool clSelfThinMort::DoMort(clTree *p_oTree, const float &fDiam, const short int &iSpecies) {
 try {
   float fRandom = clModelMath::GetRand(), //random number
         fAnnualDeathProb,    //annual probability of death
         fTimestepDeathProb;  //death probability for the entire timestep

   //If this is adult self-thinning make sure we're below the max dbh - if
   //not, exit with no death
   if (m_bIsAdult && fDiam > mp_fSelfThinMaxDbh[iSpecies])
     return false;

   fAnnualDeathProb = clModelMath::CalcPointValue(fDiam,
                                     mp_fSelfThinSlope[iSpecies],
                                     mp_fSelfThinIntercept[iSpecies]);

   //The function's linear - the result can be less than zero - make zero
   //the minimum
   if (fAnnualDeathProb < 0.0)
     fAnnualDeathProb = 0.0;

   //Compound into the death probability for the whole timestep
   fTimestepDeathProb = 1 -
           pow(1 - fAnnualDeathProb, m_fNumberYearsPerTimestep);

   if(fRandom < fTimestepDeathProb)
     return true;
   else
     return false;

 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clSelfThinMort::DoMort");
   throw(stcErr);
 }
}
//---------------------------------------------------------------------------

